"""Download and store meteorological time series (precipitation and 
evaporation) from KNMI.
"""
# %%
import json
import logging
import sys

import hydropandas as hpd
from hydropandas.io.io_pastas import _get_metadata_from_obs
from tqdm import tqdm

sys.path.insert(1, "..")
from util import get_pastastore

logging.basicConfig(level=logging.INFO)

# %% setup pastastore
with open("../tsa_settings.json", "r") as f:
    settings = json.load(f)


name = f"frontiers_{settings['freq']}"
pstore = get_pastastore(name, settings["connector_type"])
conn = pstore.conn

# %% locations oseries
oseries = conn.oseries

# %% download KNMI data

prec = hpd.ObsCollection.from_knmi(xmid=oseries.x, ymid=oseries.y,
                                   meteo_vars=["RH"], start=["1980-01-01"],
                                   ObsClass=[hpd.PrecipitationObs])

evap = hpd.ObsCollection.from_knmi(xmid=oseries.x, ymid=oseries.y,
                                   meteo_vars=["EV24"], start=["1980-01-01"],
                                   ObsClass=[hpd.EvaporationObs])

# %% write to store

for o in tqdm(prec.loc[:, "obs"].values, desc="Add precipitation series"):

    meta = _get_metadata_from_obs(o)
    meta["unit"] = "mm/day"
    # convert to mm/day
    conn.add_stress(o["RH"] * 1e3, o.name, "prec",
                    metadata=meta, overwrite=True)

for o in tqdm(evap.loc[:, "obs"].values, desc="Add evaporation series"):

    meta = _get_metadata_from_obs(o)
    meta["unit"] = "mm/day"
    # convert to mm/day
    conn.add_stress(o["EV24"] * 1e3, o.name, "evap",
                    metadata=meta, overwrite=True)
