"""Read time series for river stage from Rijkswaterstaat.

Note: For new data, these zipfiles have to be downloaded manually from 
waterinfo.rws.nl. The following locations were used in this study:
   - Arnhem
   - Driel boven
   - Driel benede
   - Pannerdense Kop
   - Pannerdense kop (note the small k, data continues where the previous stopped)
   - Grebbe
   - Dodewaard
   - Nijmegen haven
   - IJsselkop

"""
# %%
import json
import logging
import os
import sys

import hydropandas as hpd
import pandas as pd

sys.path.insert(1, "..")
from util import get_pastastore

logging.basicConfig(level=logging.INFO)

# %% setup pastastore
with open("../tsa_settings.json", "r") as f:
    settings = json.load(f)

name = f"frontiers_{settings['freq']}"
pstore = get_pastastore(name, settings["connector_type"])
conn = pstore.conn

# %% read data
datadir = "../../data"

wi2020 = hpd.ObsCollection.from_waterinfo(
    os.path.join(datadir, "river", "2020"))
wi2021 = hpd.ObsCollection.from_waterinfo(
    os.path.join(datadir, "river", "2021"),
    index_cols=["WAARNEMINGDATUM", "WAARNEMINGTIJD (MET/CET)"])

new_index = []
for i in wi2021.index:
    if i == "Pannerdense kop":
        new_index.append("Pannerdense Kop")
    else:
        new_index.append(i)
wi2021.index = new_index

# %% add data to store
for nam in wi2020.index:
    s1 = wi2020.loc[nam, "obs"]
    try:
        s2 = wi2021.loc[nam, "obs"]
        # check if 2 timeseries with same name in 2021 data
        if s2.index.size == 2:
            s2a, s2b = s2
            if nam == "Pannerdense Kop":
                s2 = pd.concat(
                    [s2a.loc[:s2b.index[0] - pd.Timedelta(minutes=1)], s2b],
                    axis=0, sort=True)
            else:
                s2 = pd.concat(
                    [s2b.loc[:s2a.index[0] - pd.Timedelta(minutes=1)], s2a],
                    axis=0, sort=True)

        s = pd.concat(
            [s1.loc[:s2.index[0] - pd.Timedelta(minutes=1)], s2], axis=0)
    except KeyError:
        s = s1

    # resample to daily data
    s = s["NUMERIEKEWAARDE"].resample("D", label='right',
                                      closed='right').mean()

    s.name = s1.meta["name"]
    if s.first_valid_index() < pd.Timestamp("1991"):
        conn.add_stress(s.to_frame(), s1.name, kind="wlvl",
                        metadata=s1.meta, overwrite=True)
