# %%
import json
import os
from datetime import timedelta

import matplotlib.pyplot as plt
import numpy as np
import pandas as pd
import pastas as ps
import pastastore as pst
from tqdm import tqdm

ps.logger.setLevel("ERROR")

# %%
with open("../tsa_settings.json", "r") as f:
    settings = json.load(f)

if settings["freq"] != "D":
    name = f"frontiers_{settings['freq']}"
else:
    name = "frontiers"
connstr = 'mongodb://localhost:27017/'

conn = pst.ArcticConnector(name, connstr)
pstore = pst.PastaStore(name, conn)

# for speedups set following to False
pstore.set_check_model_series_values(False)

# %% plot oseries
# for osname in pstore.oseries.index:
#     f, ax = ps.plots.series(head=pstore.get_oseries(osname), kde=True)
#     f.savefig(f'{osname}_series.png')
#     plt.close()

# %% split data into train/test set
train_start = "2000-01-01"
train_end = "2014-12-31"

test_start = "2015-01-01"
test_end = "2021-06-30"

# %% plot oseries timelength

intervals = {'hour': 60 * 60,
             'day': 60 * 60 * 24,
             'week': 60 * 60 * 24 * 7,
             '14 days': 60 * 60 * 24 * 14,
             '31 days': 60 * 60 * 24 * 31}

ax = pstore.plots.data_availability('oseries', intervals=intervals,
                                    figsize=(10, 20))
# plt.xlim([pd.to_datetime(train_start), pd.to_datetime(test_end)])
ax.grid(False)
ax.axvline(pd.to_datetime(test_start), linestyle='--', color='C1')

# %% mask based on time series length

# find oseries spanning at least 6 yrs in training period
oseries_tmin_tmax = pstore.get_tmin_tmax("oseries")

tdelta = pd.Timestamp(train_end) - oseries_tmin_tmax.tmin
mask = tdelta > pd.Timedelta(days=6 * 365)

# find oseries spanning at least 1 yrs in testing period
tdelta2 = oseries_tmin_tmax.tmax - pd.Timestamp(test_start)
mask2 = tdelta2 > pd.Timedelta(days=365)

# selection
model_names = oseries_tmin_tmax.loc[mask & mask2].index
oseries = pstore.get_oseries(model_names)

# %% mask number of unique days below 365

if settings["freq"] is "D":
    # find series with > 365 days with observations in train period
    mask3 = pd.Series(index=model_names, data=[len(
        oseries[ml_name][train_start:train_end].index.normalize().unique()) > 365
        for ml_name in model_names])

    # find series with > 365 days with observations in test period
    mask4 = pd.Series(index=model_names, data=[len(
        oseries[ml_name][test_start:test_end].index.normalize().unique()) > 365
        for ml_name in model_names])

    model_names = model_names[mask3 & mask4]

## %% mask timeseries with over ten gaps of 28 days
# not applied

# gaps28 = {}
# days = 28

# for mname in model_names:
#     series = oseries[mname].loc[train_start:test_end]
#     deltas = series.index.to_series().diff()[1:-1]
#     gaps = deltas[deltas > timedelta(days=days)]
#     gaps28[mname] = [len(gaps)]  # np.max(gaps)

# gaps28 = pd.DataFrame(gaps28, index=['Max Gap Length',
# f'Number of Gaps >{days} Days']).transpose().squeeze()
# gaps28.to_csv('gaps28.csv')

# mask5 = pd.DataFrame(gaps28).squeeze() < 11
# model_names = model_names[mask5]

# %%
ax = pstore.plots.data_availability('oseries', names=model_names,
                                    intervals=intervals,
                                    figsize=(10, 20))
plt.xlim([pd.to_datetime(train_start), pd.to_datetime(train_end)])
ax.grid(False)
ax.axvline(pd.to_datetime(test_start), linestyle='--', color='C1')

# %%
df = pd.read_csv('G:/My Drive/k/01projekt/frontiersin_special_issue/oseries - all.csv')

# np.save(f'model_names', df.series_sel.dropna().values) #model_names
